require ( "featMonitor_afar1" )
require ( "featMonitor_afar2" )
require ( "featMonitor_afar3" )
require ( "featMonitor_firstStageClear" )
require ( "featMonitor_allClear" )
require ( "featMonitor_allClear_noMiss" )
require ( "featMonitor_consecution1" )
require ( "featMonitor_consecution2" )
require ( "featMonitor_consecution3" )
require ( "featMonitor_charge1" )
require ( "featMonitor_charge2" )
require ( "featMonitor_charge3" )
require ( "featMonitor_noOnigiri" )
require ( "featMonitor_left1" )
require ( "featMonitor_left2" )
require ( "featMonitor_tutorialComplete" )
require ( "featMonitor_allOnigirisDelivered" )
require ( "featMonitor_allAreaClear" )
require ( "featMonitor_allAreaClear_noMiss" )
require ( "featMonitor_exploreTheGreatUnknown" )
require ( "featMonitor_recoverFromNearDeath" )

FeatWindow = {}
FeatWindow.new = function ( params )
	
	local this = {}
	this.className = "FeatWindow"
	this.params = paramas
	
	this.width = 600
	this.height = 580
	this.x = WIDTH -- 画面の右端に隠れているという位置づけ
	this.y = (HEIGHT - this.height) / 2
	
	this.isActive = false
	this.isToHide = false
	
	this.canvasForList = love.graphics.newCanvas (560, 510) -- this.width -10, this.height - 50 )
	this.canvasForListX = 20
	this.canvasForListY = 50
	this.listScrollX = 0
	this.listScrollY = 0
	this.listScrollX_offset = 50
	this.listScrollY_offset = 20
	this.scrollY_min = 0
	this.scrollY_max = 9999
	
	--=========================================================
	-- 表示へ
	--=========================================================
	this.show = function (self)
		self.isActive = true
		self.isToHide = false
		self.goalX = (WIDTH - this.width) / 2
		self.goalY = (HEIGHT - this.height) / 2

		playSound ( sounds["cursorMove"] )		
	end

	--=========================================================
	-- 非表示へ
	--=========================================================
	this.hide = function (self)
		
		self.goalX = WIDTH -- 画面の右端に隠れているという位置づけ
		self.goalY = (HEIGHT - this.height) / 2
		self.isToHide = true

		playSound ( sounds["cursorMove"] )		
	end

	--=========================================================
	-- 更新
	--=========================================================
	this.update = function (self)
		
		if ( not self.isActive ) then
			return
		end
		
		self.x, self.y = utils.Move ( self.x, self.y, self.goalX, self.goalY, 60 )
		
		if ( self.isToHide ) then
			if ( self.x == self.goalX and self.y == self.goalY ) then
				self.isActive = false
				self.isToHide = false
			end

			if ( nonRepeatButton_z ) then
				self:show()
			end
		else
			if ( nonRepeatButton_z ) then
				self:hide()
			end
		end
		
		-- 移動前に範囲を修正することで、端に到達したときに「カクッ」となって、端であることが体感できる。
		self.scrollY_max = 0
		self.scrollY_min = -1 * (#featMonitors * 50)  + self.canvasForList:getHeight() - 20
		if (self.listScrollY < self.scrollY_min) then self.listScrollY = self.scrollY_min end
		if (self.listScrollY > self.scrollY_max) then self.listScrollY = self.scrollY_max end
		
		local anaDeg, anaDirection = getStickDeg()
		--print (anaDeg, anaDirection)
		
		if ( love.keyboard.isDown("up")   or gamePad["up"] or anaDirection == "UP" ) then self.listScrollY = self.listScrollY + 5 end
		if ( love.keyboard.isDown("down") or gamePad["down"] or anaDirection == "DOWN") then self.listScrollY = self.listScrollY - 5 end
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function (self)

		if ( not self.isActive ) then
			return
		end
	
		local r,g,b,a = love.graphics.getColor()
		local currentFont = love.graphics.getFont()
		
		local outlineWidth = 4
		love.graphics.setColor ( 0.7, 0.7, 0.7, 1 )
		love.graphics.rectangle( "fill", self.x - outlineWidth, self.y - outlineWidth, self.width + outlineWidth*2, self.height + outlineWidth*2, 10)

		love.graphics.setColor ( 0,0,0.2,1 )
		love.graphics.rectangle( "fill", self.x, self.y, self.width, self.height, 10)
		
		local font = fonts.score
		love.graphics.setFont ( font )
		love.graphics.setColor ( 1,1,1,1 )
		local caption = "<< 実績 >>" 
		love.graphics.print ( caption, self.x + self.width/2 - font:getWidth(caption)/2, self.y + 15 )

		love.graphics.setFont ( fonts.featDescription )
		if ( self.listScrollY < self.scrollY_max ) then
			love.graphics.print ( "Scroll up[↑]", self.x + 475, self.y + 30 )
		end
		if ( self.listScrollY > self.scrollY_min ) then
			love.graphics.print ( "Scroll down[↓]", self.x + 460, self.y + 560 )
		end
		love.graphics.setFont ( font )
		
		local currentCanvas = love.graphics.getCanvas()
		love.graphics.setCanvas ( self.canvasForList )
		love.graphics.clear(0,0,0,0)
		
		local monitorIconOffsetX = self.listScrollX + self.listScrollX_offset-- 70
		local monitorIconOffsetY = self.listScrollY + self.listScrollY_offset --60
		
		local isComp = true
		for i = 1, #featMonitors, 1 do
			
			self:drawFeatInfo ( featMonitors[i], monitorIconOffsetX, monitorIconOffsetY + (i-1)*50 )
			--featMonitors[i]:draw ( self.x + monitorIconOffsetX, self.y + monitorIconOffsetY + (i-1)*50 )
			
			if ( not featMonitors[i].isAchieved ) then isComp = false end
		end
		
		love.graphics.setCanvas (currentCanvas)
		love.graphics.setColor(0,0,0,1)
		love.graphics.rectangle ( "fill", self.x + self.canvasForListX, self.y + self.canvasForListY, self.canvasForList:getWidth(), self.canvasForList:getHeight() )
		love.graphics.setColor(1,1,1,1)
		love.graphics.draw ( self.canvasForList, self.x + self.canvasForListX, self.y + self.canvasForListY )
		
		-- コンプ表示
		if ( isComp ) then
			love.graphics.setColor(1,1,0,1)
			love.graphics.print ( "Complete!!", self.x + 2, self.y + 30, math.rad(-15) )
		end
		
		love.graphics.setFont(currentFont)
		love.graphics.setColor ( r,g,b,a )
	end
	
	
	--=========================================================
	-- 描画＿個別の実績をアイコンとして
	--=========================================================
	this.drawFeatInfo = function (self, featMonitor, x, y)
	
		local r,g,b,a = love.graphics.getColor()
		local currentFont = love.graphics.getFont()
		local currentLineWidth  = love.graphics.getLineWidth()
		
		local font = fonts.default
		love.graphics.setFont(font)
		love.graphics.setLineWidth(2)
		local cornerRound = 5
		
		local featMonitor_width  = 500
		local featMonitor_height = 40
				
		love.graphics.setColor ( 0,0,0,1 )
		love.graphics.rectangle( "fill", x, y, featMonitor_width, featMonitor_height, cornerRound)
		love.graphics.setColor ( 1,1,1,1 )
		love.graphics.rectangle( "line", x, y, featMonitor_width, featMonitor_height, cornerRound)
		
		love.graphics.setColor ( 0,0,0,1 )
		love.graphics.rectangle( "fill", x+10, y-8, font:getWidth(featMonitor.featName)+4, font:getHeight())
		love.graphics.setColor ( 1,1,1,1 )
		love.graphics.print ( featMonitor.featName, x + 12, y - 8 )


		love.graphics.setFont(fonts.featDescription)
		love.graphics.print ( featMonitor.description, x + 20, y + 14 )

		love.graphics.setFont(fonts.featState)
		
		local state = "[未]"
		love.graphics.setColor ( 0.5, 0.5, 0.5,1 )
		
		if ( featMonitor.isAchieved ) then
			state = "[★]"
			love.graphics.setColor ( 1,1,0,1 )
		end
		love.graphics.print ( state, x - 45, y + 2 )
		
		love.graphics.setFont(fonts.featDescription)
		love.graphics.print ( "達成", x - 40, y + 24)

		love.graphics.setLineWidth(currentLineWidth)
		love.graphics.setFont(currentFont)
		love.graphics.setColor ( r,g,b,a )
	end
	
	return this
end
 

