
scripts = require("scripts")

StoryDemo = {}
StoryDemo.new = function ( script )

	local this = {}
	this.className = "StoryDemo"
	this.params = paramas

	this.script = script -- 処理の基本構造は procName と params
	this.procs = {}
	this.canvas = love.graphics.newCanvas ( WIDTH, HEIGHT )
	
	-- 進行につれて変化するもの
	this.frameCount = 0
	this.backgroundColor = {r = 0, g = 0, b = 0, a = 1}
	this.images = {}
	this.texts  = {}
	this.sounds = {}
	this.fonts  = {}
	this.repeatProcs = {}
	this.isEnd = false
	this.translateX = 0
	this.translateY = 0
	
	this.defaultSettings_text = {
		x = WIDTH/2, y = HEIGHT/2, displayWaitMax = 1, lineLength = 20, fontId = "default"
	}
	
	--=========================================================
	-- 初期設定
	--=========================================================
	this.init = function (self)
		
		self.procs [ "registImage"        ] = self.registImage
		self.procs [ "image_setImage"     ] = self.image_setImage
		self.procs [ "image_setPosition"  ] = self.image_setPosition
		self.procs [ "image_move"         ] = self.image_move
		self.procs [ "image_setBrightness"] = self.image_setBrightness
		self.procs [ "image_setAlpha"     ] = self.image_setAlpha
		self.procs [ "image_setAngle"     ] = self.image_setAngle
		self.procs [ "image_setScale"     ] = self.image_setScale
		self.procs [ "image_setShake"     ] = self.image_setShake
		self.procs [ "image_show"         ] = self.image_show
		self.procs [ "image_hide"         ] = self.image_hide

		self.procs [ "registFont"         ] = self.registFont
		self.procs [ "registText"         ] = self.registText
		self.procs [ "text_setPosition"   ] = self.text_setPosition
		self.procs [ "text_move"          ] = self.text_move
		self.procs [ "text_setDisplayWait"] = self.text_setDisplayWait
		self.procs [ "text_setLineSpace"  ] = self.text_setLineSpace
		self.procs [ "text_setColor"      ] = self.text_setColor
		self.procs [ "text_show"          ] = self.text_show
		self.procs [ "text_hide"          ] = self.text_hide
		self.procs [ "text_setDefaultSettings"] = self.text_setDefaultSettings
		
		self.procs [ "registSound"        ] = self.registSound
		self.procs [ "sound_play"         ] = self.sound_play
		self.procs [ "sound_stop"         ] = self.sound_stop
		self.procs [ "sound_setLoop"      ] = self.sound_setLoop
		self.procs [ "sound_fadeout"      ] = self.sound_fadeout
		self.procs [ "sound_setVolume"    ] = self.sound_setVolume

		self.procs [ "registRepeatProc"   ] = self.registRepeatProc
		self.procs [ "deleteRepeatProc"   ] = self.deleteRepeatProc

		self.procs [ "setBackgroundColor" ] = self.setBackgroundColor
		self.procs [ "endDemo"            ] = self.endDemo
		self.procs [ "nextPage"           ] = self.toNextScript

		self.procs [ "draw_setTranslate"  ] = self.draw_setTranslate
		self.procs [ "custom"             ] = self.custom
		
		self:reset()
	end
	
	--=========================================================
	-- 終了処理
	--=========================================================
	this.destruct = function (self)
		self:sound_stopAll()
	end
	
	--=========================================================
	-- 進行状況をリセット
	--=========================================================
	this.reset = function (self)
	
		-- 進行につれて変化するもの
		self.frameCount = 0
		self.backgroundColor = {r = 0, g = 0, b = 0, a = 1}
		self.images = {}
		self.texts  = {}
		--self.sounds = {}
		self.fonts  = {}
		self.repeatProcs = {}
		self.isEnd = false
		self.translateX = 0
		self.translateY = 0		
	end
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function (self)
		
		if ( self.isEnd ) then
			--return
		end
		
		self.frameCount = self.frameCount + 1
		
		local proc = self.script [ self.frameCount ]
		for i = 1, #self.script, 1 do
			
			if ( self.frameCount == self.script[i].frame ) then
			
				local procFunction = self.procs[ self.script[i].procName ]
				if ( procFunction ) then
					procFunction ( self, self.script[i] )
				end
			end
			
		end
		
		-- 画像オブジェクト更新
		for i = 1, #self.images, 1 do
			
			-- 移動
			self.images[i].x, self.images[i].y = utils.Move (self.images[i].x, self.images[i].y, self.images[i].goalX, self.images[i].goalY, self.images[i].speed) 
			
		end
		
		-- テキストオブジェクト更新
		for i = 1, #self.texts, 1 do
			
			if ( self.texts[i].isShown ) then
			
				-- 移動
				self.texts[i].x, self.texts[i].y = utils.Move (self.texts[i].x, self.texts[i].y, self.texts[i].goalX, self.texts[i].goalY, self.texts[i].speed) 
				
				-- 文字を進める
				if ( self.texts[i].displayWaitMax < 0 ) then -- 最初から全部表示するという意味
					self.texts[i].currentIndex = #self.texts[i].charList
				else
					
					if ( self.texts[i].currentDisplayWait >= self.texts[i].displayWaitMax ) then
					
						self.texts[i].currentIndex = self.texts[i].currentIndex + 1
						self.texts[i].currentDisplayWait = 0
						
						if ( self.texts[i].currentIndex > #self.texts[i].charList ) then
							self.texts[i].currentIndex = #self.texts[i].charList
						end
					else
						self.texts[i].currentDisplayWait = self.texts[i].currentDisplayWait + 1
					end
				end
			end
		end
		
		-- 音声オブジェクト更新
		for i = 1, #self.sounds, 1 do
			
			-- フェードアウト
			if ( self.sounds[i].fadeoutPace ) then
				
				print (self.sounds[i].source:getVolume())
				
				self.sounds[i].source:setVolume ( utils.Approach ( self.sounds[i].source:getVolume(), 0, self.sounds[i].fadeoutPace ) )
				
				if ( self.sounds[i].source:getVolume() == 0 ) then
					self.sounds[i].fadeoutPace = nil
					self.sounds[i].source:setVolume ( 1.0 )
					self.sounds[i].source:stop()
				end
			end
			
		end

		-- 繰り返し処理オブジェクト更新
		for i = 1, #self.repeatProcs, 1 do
			
			if ( self.repeatProcs[i].repeatWait >= self.repeatProcs[i].repeatWaitMax ) then
				self.repeatProcs[i].repeatWait = 0
				
				-- 実行
				local procFunction = self.procs[ self.repeatProcs[i].proc.procName ]
				if ( procFunction ) then
					procFunction ( self, self.repeatProcs[i].proc )
					--print ("繰り返し処理実行")
				end
				
			else
				self.repeatProcs[i].repeatWait = self.repeatProcs[i].repeatWait + 1
			end
			
		end
	end
	
	--=========================================================
	-- 画像取得
	--=========================================================
	this.getImage = function (self, id)
		local result = nil
			for i = 1, #self.images, 1 do
				if ( self.images[i].id == id ) then
					result = self.images[i]
					break
				end
			end
		return result
	end
	
	--=========================================================
	-- テキスト取得
	--=========================================================
	this.getText = function (self, id)
		local result = nil
			for i = 1, #self.texts, 1 do
				if ( self.texts[i].id == id ) then
					result = self.texts[i]
					break
				end
			end
		return result
	end
	
	--=========================================================
	-- サウンド取得
	--=========================================================
	this.getSound = function (self, id)
		local result = nil
			for i = 1, #self.sounds, 1 do
				if ( self.sounds[i].id == id ) then
					result = self.sounds[i]
					break
				end
			end
		return result
	end

	--=========================================================
	-- 繰り返し処理取得
	--=========================================================
	this.getRepeatProc = function (self, id)
		local result = nil
			for i = 1, #self.repeatProc, 1 do
				if ( self.repeatProc[i].id == id ) then
					result = self.repeatProc[i]
					break
				end
			end
		return result
	end
	
	--=========================================================
	-- 処理：別スクリプトへ移動
	--=========================================================
	this.toNextScript = function (self, params)
		print ("処理：次のページへ", params.scriptName )
		self.script = scripts[params.scriptName]
		self:reset()
	end
	
	--=========================================================
	-- 処理：背景色設定
	--=========================================================
	this.setBackgroundColor = function (self, params)
		print ("処理：背景色設定", params.r, params.g, params.b, params.a)
		self.backgroundColor.r = params.r 
		self.backgroundColor.g = params.g 
		self.backgroundColor.b = params.b 
		self.backgroundColor.a = params.a 
	end

	--=========================================================
	-- 処理：終了フラグ
	--=========================================================
	this.endDemo = function (self, params)
		print ("処理：終了フラグ")
		self.isEnd = true
	end
	
	--=========================================================
	-- 処理：画像登録
	--=========================================================
	this.registImage = function (self, params)
	
		print("処理：画像登録", params.id, params.file)
		local image = { 
			id = params.id,
			image = love.graphics.newImage ( IMAGE_DIR .. "/" .. params.file ),
				--{mipmaps = true, linear = true, dpiscale = 1} ),
			x = WIDTH/2, y = HEIGHT/2,
			goalX = WIDTH/2, goalY = HEIGHT/2, speed = 1,
			brightness = 1.0,
			alpha = 1.0,
			angle = 0,
			scaleW = 1.0, scaleH = 1.0,
			shake=0, 
			isShown = false
		}
		self.images [ #self.images + 1 ] = image
		
		--image.image:setMipmapFilter("nearest", 1)
		
	end
	this.image_setImage = function (self, params)
		print("処理：画像再設定", params.id, params.file)
		local image = self:getImage(params.id)
		image.image = love.graphics.newImage ( IMAGE_DIR .. "/" .. params.file )
	end
	this.image_setPosition = function (self, params)
		print("処理：画像位置設定", params.id, params.x, params.y, params.otherImage)
		local image = self:getImage(params.id)
		
		if ( params.otherImage ) then
			print ("別画像の位置に合わせる：", params.otherImage)
			local otherImage = self:getImage(params.otherImage)
			params.x = otherImage.x
			params.y = otherImage.y
		end
		
		image.x = tonumber(params.x)
		image.y = tonumber(params.y)
		image.goalX = image.x
		image.goalY = image.y
	end
	this.image_move = function (self, params)
		print("処理：画像移動", params.id, params.toX, params.toY, params.speed)
		local image = self:getImage(params.id)
		image.goalX = tonumber(params.toX)
		image.goalY = tonumber(params.toY)
		image.speed = tonumber(params.speed)
	end
	this.image_setBrightness = function (self, params)
		print("処理：画像明度設定", params.id, params.brightness)
		local image = self:getImage(params.id)
		image.brightness = tonumber(params.brightness)
	end
	this.image_setAlpha = function (self, params)
		print("処理：画像不透明度設定", params.id, params.alpha)
		local image = self:getImage(params.id)
		image.alpha = tonumber(params.alpha)
	end
	this.image_setAngle = function (self, params)
		print("処理：画像回転角度設定", params.id, params.angle)
		local image = self:getImage(params.id)
		image.angle = tonumber(params.angle)
	end
	this.image_setScale = function (self, params)
		print("処理：画像拡大率設定", params.id, params.scaleW, params.scaleH)
		local image = self:getImage(params.id)
		image.scaleW = tonumber(params.scaleW)
		image.scaleH = tonumber(params.scaleH)
	end
	this.image_setShake = function (self, params)
		print("処理：画像の振動を設定", params.id, params.shake)
		local image = self:getImage(params.id)
		image.shake = tonumber(params.shake)
	end
	this.image_show = function (self, params)
		print("処理：画像表示", params.id)
		local image = self:getImage(params.id)
		image.isShown = true
	end
	this.image_hide = function (self, params)
		print("処理：画像非表示", params.id)
		local image = self:getImage(params.id)
		image.isShown = false
	end
	
	--=========================================================
	-- 処理：フォントを登録
	--=========================================================
	this.registFont = function (self, params)
		print ("処理：フォント登録", params.file, params.size)
		self.fonts [ params.id ] = love.graphics.newFont ( params.file, params.size)
	end	
	
	--=========================================================
	-- 処理：テキスト登録
	--=========================================================
	this.text_setDefaultSettings = function (self, params)
		print ( "処理：テキストのデフォルト設定を設定" )
		
		self.defaultSettings_text.x              = params.x
		self.defaultSettings_text.y              = params.y
		self.defaultSettings_text.displayWaitMax = params.displayWait
		self.defaultSettings_text.lineLength     = params.lineLength
		self.defaultSettings_text.fontId         = params.fontId
		
	end
	
	--=========================================================
	-- 処理：テキスト登録
	--=========================================================
	this.registText = function (self, params)
	
		print("処理：テキスト登録", params.id, params.text, params.fontId, params.lineLength)
		
		if ( params.lineLength == nil ) then params.lineLength = self.defaultSettings_text.lineLength end
		if ( params.fontId     == nil ) then params.fontId     = self.defaultSettings_text.fontId     end
		
		local text = { 
			id = params.id,
			text = params.text,
			x = self.defaultSettings_text.x, y = self.defaultSettings_text.y,
			goalX = self.defaultSettings_text.x, goalY = self.defaultSettings_text.y, speed = 1,
			r = 1,g = 1,b = 1,a = 1,
			isShown = false,
			fontId = params.fontId,
			
			currentIndex       = 0,
			displayWaitMax     = self.defaultSettings_text.displayWaitMax,
			currentDisplayWait = self.defaultSettings_text.displayWaitMax,
			lineLength         = params.lineLength,
			
		}
		
		text.lineSpace = math.floor(self.fonts[text.fontId]:getHeight()/2)
		text.charList = textToCharList ( text.text )
		text.lineList = listToLists ( text.charList, text.lineLength )
		
		-- 内容確認
		for i = 1, #text.lineList, 1 do
			local str = ""
			for j = 1, #text.lineList[i], 1 do
				str = str .. text.lineList[i][j]
			end
			print("> " .. str)
		end
		
		self.texts [ #self.texts + 1 ] = text
		
	end
	this.text_setPosition = function (self, params)
		print("処理：テキスト位置設定", params.id, params.x, params.y)
		local text = self:getText(params.id)
		text.x = tonumber(params.x)
		text.y = tonumber(params.y)
		text.goalX = text.x
		text.goalY = text.y
	end
	this.text_setDisplayWait = function (self, params)
		print("処理：テキスト表示速度設定", params.id, params.displayWait)
		local text = self:getText(params.id)
		text.displayWaitMax = tonumber(params.displayWait)
	end
	this.text_setLineSpace = function (self, params)
		print("処理：テキスト行間設定", params.id, params.lineSpace)
		local text = self:getText(params.id)
		text.lineSpace = tonumber(params.lineSpace)
	end
	this.text_setColor = function (self, params)
		print("処理：テキスト色設定", params.id, params.r, params.g, params.b, params.a)
		local text = self:getText(params.id)
		text.r = tonumber(params.r)
		text.g = tonumber(params.g)
		text.b = tonumber(params.b)
		text.a = tonumber(params.a)
	end
	this.text_show = function (self, params)
		print("処理：テキスト表示", params.id)
		local text = self:getText(params.id)
		text.isShown = true
	end
	this.text_hide = function (self, params)
		print("処理：画像非表示", params.id)
		local text = self:getText(params.id)
		text.isShown = false
	end
	
	
	--=========================================================
	-- 処理：音声を登録
	--=========================================================
	this.registSound = function (self, params)
	
		--print("処理：音声登録", params.id, params.file, params.type)
		print("処理：音声登録", params.id )
		local sound = { 
			id = params.id,
			--source = love.audio.newSource ( AUDIO_DIR .."/"..params.file, params.type ),
			source = sounds[params.id],
			fadeoutPace = nil
		}
		
		print ( sound.source, params.id )
		
		self.sounds [#self.sounds + 1] = sound
	end
	this.sound_play = function (self, params)
		print("処理：音声再生", params.id)
		local sound = self:getSound(params.id)
		sound.source:stop()
		love.audio.play ( sound.source )
	end
	this.sound_stop = function (self, params)
		print("処理：音声停止", params.id)
		local sound = self:getSound(params.id)
		sound.source:stop()
	end
	this.sound_stopAll = function (self)
		print("処理：全音声停止", #self.sounds )
		for i = 1, #self.sounds, 1 do
			self.sounds[i].source:stop()
		end
	end
	this.sound_setLoop = function (self, params)
		print("処理：音声ループ設定", params.id)
		local sound = self:getSound(params.id)
		sound.source:setLooping ( params.loop == "true" )
	end
	this.sound_fadeout = function (self, params)
		print("処理：音声フェードアウト", params.id, params.pace)
		local sound = self:getSound(params.id)
		sound.fadeoutPace = tonumber(params.pace)
	end
	this.sound_setVolume = function (self, params)
		print("処理：音量設定", params.id)
		local sound = self:getSound(params.id)
		sound.source:setVolume ( params.volume )
		sound.fadeoutPace = nil
	end
	
	--=========================================================
	-- 処理：繰り返し処理を登録
	--=========================================================
	this.registRepeatProc = function (self, params)
		
		print ("繰り返し処理を登録", params.id, params.proc.procName)
		
		local repeatProc = {
			id = params.id,
			repeatWaitMax = params.repeatWait,
			repeatWait = 0,
			proc = params.proc
		}
		
		self.repeatProcs [ #self.repeatProcs + 1 ] = repeatProc
		
	end
	--=========================================================
	-- 処理：繰り返し処理を削除
	--=========================================================
	this.deleteRepeatProc = function (self, params)
		
		print ("繰り返し処理を削除", params.id)
		
		for i = 1, #self.repeatProcs, 1 do
			if ( self.repeatProcs[i].id == params.id ) then
				table.remove ( self.repeatProcs, i )
				break
			end
		end
	end
	--=========================================================
	-- 処理：繰り返し処理を削除
	--=========================================================
	this.draw_setTranslate = function (self, params)
		
		print ("描画位置を設定", params.translateX, params.translateY)
		
		self.translateX = params.translateX
		self.translateY = params.translateY
		
	end
	
	--=========================================================
	-- 処理：ユーザー定義処理
	--=========================================================
	this.custom = function (self, params)
		
		print ("ユーザー定義処理を実行")
		
		params.func(self)
		
	end
	
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function (self)
	
		love.graphics.push()
		love.graphics.translate ( self.translateX, self.translateY )
		
		local r,g,b,a = love.graphics.getColor()
		local currentCanvas = love.graphics.getCanvas()
		local currentFont = love.graphics.getFont()
		
		love.graphics.setCanvas( self.canvas )
		
		love.graphics.clear ( self.backgroundColor.r, self.backgroundColor.g, self.backgroundColor.b, self.backgroundColor.a )
		
		-- 画像表示
		for i = 1, #self.images, 1 do
			
			if ( self.images[i].isShown ) then
				
				-- 色関連の設定
				local brightness = self.images[i].brightness
				love.graphics.setColor ( brightness,brightness,brightness, self.images[i].alpha)
				
				-- 各種設定と共に描画
				love.graphics.draw ( self.images[i].image, math.floor(self.images[i].x) + math.random(-1* self.images[i].shake, self.images[i].shake), math.floor(self.images[i].y) + math.random(-1* self.images[i].shake, self.images[i].shake), 
					math.rad(self.images[i].angle), self.images[i].scaleW, self.images[i].scaleH, math.floor(self.images[i].image:getWidth()/2), math.floor(self.images[i].image:getHeight()/2) )
				
				-- 画像がテキストを持っている場合、描画
				if ( self.images[i].textObj ) then
					local text = self.images[i].textObj.text
					local x = self.images[i].textObj.x
					local y = self.images[i].textObj.y
					love.graphics.print ( text, x, y )
				end
			
			end
		end
		
		-- テキスト表示
		for i = 1, #self.texts, 1 do
			
			if ( self.texts[i].isShown ) then
				
				-- 色関連の設定
				local r = self.texts[i].r
				local g = self.texts[i].g
				local b = self.texts[i].b
				local a = self.texts[i].a
				love.graphics.setColor ( r,g,b,a )
				
				-- 今回表示する分
				local displayLine = {}
				local totalCharNum = 0
				
				for n = 1, #self.texts[i].lineList, 1 do
					if ( totalCharNum >= self.texts[i].currentIndex ) then break end
					
					local line = ""
					
					for s = 1, #self.texts[i].lineList[n], 1 do
						if ( totalCharNum >= self.texts[i].currentIndex ) then break end
						
						line = line .. self.texts[i].lineList[n][s]
						totalCharNum = totalCharNum + 1
					end
					displayLine [#displayLine + 1] = line
					
				end
				
				-- フォントを設定して、表示
				local font = self.fonts [ self.texts[i].fontId ]
				love.graphics.setFont ( font )
				
				for n = 1, #displayLine, 1 do
					local y = self.texts[i].y + (n-1) * (font:getHeight() + self.texts[i].lineSpace)
					love.graphics.print ( displayLine[n], self.texts[i].x, y  )
				end
				
			end
		end
		
		
		love.graphics.setCanvas( currentCanvas )

		love.graphics.setColor ( 1,1,1,1 )
		love.graphics.setFont(currentFont)

		love.graphics.draw ( self.canvas, 0, 0 )

		love.graphics.setColor ( r,g,b,a )
		
		love.graphics.pop()
	end
	
	
	this:init()
	
	return this
end
 

