
--=========================================================
-- 実績の条件判定の内部フラグをリセット
--=========================================================
function resetAchievementsFlags ()
	
	for i = 1, #featMonitors, 1 do
		if ( featMonitors[i].reset ) then
			featMonitors[i]:reset()
		end
	end
	
end

--=========================================================
-- 残機をリセット
--=========================================================
function left_reset ( n )
	
	left = {}
	for i = 1, n, 1 do
		left[#left + 1] = Left.new()
	end
end

--=========================================================
-- 残機を１つ取り出す
--=========================================================
function left_get ()
	
	if ( #left == 0 ) then
		return nil
	end
	
	local result = left[1]
	table.remove ( left, 1 )
	return result
end

--=========================================================
-- 残機を１つ追加
--=========================================================
function left_add ()
	left[#left + 1] = Left.new()
end

--=========================================================
-- 残機にエネルギーを追加
--=========================================================
function left_addEnergy ( energy )
	
	for i = 1, #left, 1 do
		
		energy = left[i]:addEnergy(energy)
		
		if ( energy == 0 ) then
			break
		end
	end
	
end


--=========================================================
-- 「返答」を追加
--=========================================================
function addReaction ( x, y, direction, type )
	
	local reaction = Reaction.new()
	reaction.x = x
	reaction.y = y
	reaction.moveDirection = direction
	reaction.type = type
	
	reactions [ #reactions + 1 ] = reaction
		
end

--=========================================================
-- 「未知」を追加
--=========================================================
function addUnknown (num)
	
	if (not num) then num = 1 end
	
	for i = 1, num, 1 do
		local unknown = Unknown.new()
		unknown:init()
		unknowns [ #unknowns + 1 ] = unknown
		
		if ( mode == "story" and stage == 1) then
			unknown.type = "A"
		end
	end
	
end
--=========================================================
-- 大型「未知」を追加
--=========================================================
function addUnknown_big (num)
	
	if (not num) then num = 1 end
	
	for i = 1, num, 1 do
		local unknown = Unknown_big.new()
		unknown:init()
		unknowns [ #unknowns + 1 ] = unknown
	end
	
end

--=========================================================
-- 岩を追加
--=========================================================
function addRock (num)
	
	if (not num) then num = 1 end
	
	for i = 1, num, 1 do
		local rock = Rock.new()
		rocks [ #rocks + 1 ] = rock
	end
		
end

--=========================================================
-- アイテム追加
--=========================================================
function addItem ( itemId, x, y )
	
	local item = Item.new ( itemId )
	items [ #items + 1 ] = item
	
	if ( x ~= nil ) then
		item.x = x
	end
	if ( y ~= nil ) then
		item.y = y
	end
		
end

--=========================================================
-- 新エリア到達メッセージ
--=========================================================
function showNewAreaReachMessage ( text )

	newAreaReachMessage = text
	newAreaReachMessageX = canvas_field:getWidth() 
	playSound ( sounds["newArea"] )
end

--=========================================================
-- おにぎりを発射
--=========================================================
function popGoodWill ()
	
	local result = true --発射できたかどうか
	
	if ( goodWillLeft > 0 ) then
	
		local goodWill = GoodWill.new()
		
		goodWill.x = playerMachine.x
		goodWill.y = playerMachine.y
		
		goodWills [ #goodWills + 1 ] = goodWill
		
		goodWillLeft = goodWillLeft - 1
		
		playSound ( sounds["goodWill"] )
		
	else
		print ("発射できない")
		result = false
		playSound ( sounds["empty"] )
	end
	
	return result
end

--=========================================================
-- エフェクトメッセージを追加
--=========================================================
function addEffectMessage ( text, x, y, alphaDecrement )
	
	local effectMessage = EffectMessage.new ( {
		text = text,
		x = x,
		y = y,
		font = fonts.score
	} )
	
	if ( alphaDecrement ) then
		effectMessage.alphaDecrement = alphaDecrement
	end
	
	effectMessages [ #effectMessages + 1 ] = effectMessage
		
end

--=========================================================
-- 実績モニターを取得
--=========================================================
function getFeatMonitor ( id ) 
	
	local result = nil
	
	for i = 1, #featMonitors, 1 do
		if ( featMonitors[i].featId == id ) then
			result = featMonitors[i]
			break
		end
	end
	
	return result
end

--=========================================================
-- 実績達成メッセージを表示 
--=========================================================
function setFeatAchievedMessage ( feat )
	
	local featAchievedMessage = {
		
		count = 100,
		message = "実績「" ..feat.featName .. "」を達成！",

		update = function (self)
			if (self.isEnd) then
				return
			end
			self.count = self.count - 1
		
		end,
		
		draw = function (self, index)
		
			if (self.isEnd) then
				return
			end
			
			local colors = {
				{1,0,0}, {0,1,0}, {0,0,1},{ 1,1,0 }, {1,1,1}
			}
			local font = fonts.areaMessage
			
			love.graphics.setColor ( colors [ (math.ceil(self.count/5))%(#colors) + 1] )
			love.graphics.setFont(font)
			love.graphics.print ( self.message, 10, HEIGHT - font:getHeight() - 2 - font:getHeight() * (index-1) )
			
			if ( self.count < 0 ) then
				self.isEnd = true
			end
		end
	}
	
	featAchievedMessages [ #featAchievedMessages + 1 ] = featAchievedMessage
	playSound ( sounds["achieved"] )

end

--=========================================================
-- 岩の出現頻度と個数を取得 
--=========================================================
function getRockData() 

	local rockFreq = 0
	local rockNum = 0
	
	if ( mode == "story" ) then
		
		local s = stage -- 敢えて受けておくとデバッグしやすい
		local initValue = leftToGoalInit
		if ( s == 1 ) then
			initValue = leftToGoalInit_first
		end
		
		local rockDatas_forFirstHalf = {
			{99999, 0}, 
			{100, 2},
			{70, 3},
			{50, 4},
			{40, 4}
		}
		local rockDatas_forSecondHalf = {
			{200, 1}, 
			{70, 2},
			{50, 3},
			{40, 4},
			{30, 5}
		}
		
		-- ステージの前半か後半かで分ける
		if ( leftToGoal > initValue/2 ) then
			rockFreq = rockDatas_forFirstHalf [ s ] [ 1 ]
			rockNum = rockDatas_forFirstHalf [ s ] [ 2 ]
		else
			rockFreq = rockDatas_forSecondHalf [ s ] [ 1 ]
			rockNum = rockDatas_forSecondHalf [ s ] [ 2 ]
		end
		
		--rockFreq = 100
		--rockNum = 1
		
	elseif ( mode == "endless" ) then
		
		rockNum = areaA
		
		local rockFreqTable = { 200, 100, 50, 25 }
		rockFreq = rockFreqTable [ areaB ]
		
	end
	
	-- デバッグ用の仮
--	local rockFreq = 50
--	local rockNum = math.random(1,2)
	
	return rockFreq,rockNum
end

--=========================================================
-- 三項演算子
--=========================================================
function ternary ( condition, a, b)
	
	local result = a
	
	if ( condition ) then 
		result = a 
	else
		result = b
	end
	
	return result
	
end


--=========================================================
-- サウンドを鳴らす
--=========================================================
volumeSOUND = 1.0
function playSound ( soundSource ) 

	if ( soundSource ) then
		
		soundSource:setVolume ( volumeSOUND )
		
		if ( soundSource:isPlaying() ) then
			soundSource:stop()
		end
		love.audio.play ( soundSource )	
	
	else
		print ( "soundSource is null." )
	end
	
end

--=========================================================
-- BGMを鳴らす
--=========================================================
currentPlayingBGM = nil
function playBGM ( soundSource ) 
	
	if ( currentPlayingBGM ) then
		currentPlayingBGM:stop()
	end
	
	love.audio.play ( soundSource )	
	
	-- BGMなのでループ設定
	soundSource:setLooping(true)
	
	-- 音量
	--soundSource:setVolume ( volumeBGM )
	
	-- 「今のBGM」を更新
	currentPlayingBGM = soundSource
end
--=========================================================
-- BGMを止める
--=========================================================
function stopBGM () 
	
	if ( currentPlayingBGM ) then
		currentPlayingBGM:stop()
	end
end

--=========================================================
-- UTFの文字列を１文字ずつの配列に変換
--=========================================================
function textToCharList ( text ) 
	
	local result = {}	
	
	local currentOffset = 0
	
	for i = 1, utf8.len(text), 1 do

		local nextOffset = utf8.offset ( text, i + 1 )
		local char = string.sub ( text, currentOffset, nextOffset-1 )
		
		currentOffset = nextOffset
		
		result [ #result + 1 ] = char
	end
	
	return result
end

--=========================================================
-- リストを指定要素数で分割したリストにする
--=========================================================
function listToLists ( list, length )
	
	local result = {}
	
	local subList = {}
	for i = 1, #list, 1 do
		
		if ( (#subList >= length and (list[i] ~= "。" and list[i] ~= "、" and list[i] ~= "？" and list[i] ~= "！" and list[i] ~= "」" and list[i] ~= "）")) or list[i] == "\n" ) then
			result [#result + 1] = subList
			subList = {}
		end
		
		if ( not (list[i] == "\n") )then
			subList[#subList + 1] = list[i]
		end
	end
	
	-- 最後に余った分を回収
	if ( #subList > 0 ) then
		result [ #result + 1 ] = subList
	end
	
	
	return result
end

--=========================================================
-- 縁取りつきで文字を表示
--=========================================================
function printWithOutline ( text, x, y, font, outlineWidth )
	
	if ( outlineWidth == nil ) then outlineWidth = 1 end
	local outlineColor = {0,0,0,1}
	local r,g,b,a = love.graphics.getColor()
	local currentFont = love.graphics.getFont()

	love.graphics.setFont ( font )
	
	love.graphics.setColor(outlineColor)
	for oy = -outlineWidth, outlineWidth, 1 do
		for ox = -outlineWidth, outlineWidth, 1 do
			love.graphics.print ( text, x + ox, y + oy )
		end
	end
	
	love.graphics.setColor (r,g,b,a)
	love.graphics.print ( text, x, y )

	love.graphics.setFont ( currentFont )
end













 


